xquery version "3.1";
import module namespace helper = "identification-helper" at "identification-helper.xqm";
declare variable $model external := "0->1 1->2 2->3 3->4 0<->1 0<->2 0<->3 0<->4";
declare variable $indexOffset as xs:integer external := 0;
declare variable $edgesInput := 
  if ($model => contains("<->")) then $model
  else if ($model => starts-with("[[")) then helper:json-model-to-pretty-edge-string($model)
  else error(xs:QName("model"), "invalid model: " || $model);
declare variable $first-node-index := min((min($edges-ids) + $indexOffset, 1) );
declare variable $n external := max($edges-ids) + 1 - $first-node-index + $indexOffset; 
declare variable $outputFormat external := "maxima"; (: "debug" :)


declare variable $edges := (tokenize($edgesInput));
declare variable $edges-ids := (pxp:extract($edgesInput, "[0-9]+", 0, "*")!xs:integer(.));
declare variable $last-node-index := $n - 1 + $first-node-index;
declare variable $ids := $first-node-index to $last-node-index;

declare variable $name-separator := if ($n < 10) then "" else switch ($outputFormat) case "maxima" return "t" default return "_";
declare variable $name-sigma-prefix := switch ($outputFormat) case "maxima" return "s_" default return "s";
declare variable $name-omega-prefix := switch ($outputFormat) case "maxima" return "w_" default return "w";
declare variable $name-lambda-prefix := switch ($outputFormat) case "maxima" return "l_" default return "l";


declare function local:edge-list-to-maps($es){
   (let $combined := map:merge($es, map {"duplicates": "combine"})
  return map:merge( for $k in map:keys($combined) 
  return map {$k: map:merge($combined($k) ! map{.: true()})}
  )
  )
};

declare variable $directed-map := local:edge-list-to-maps(
  for $e in $edges return
  if (contains($e, "->") and not(contains($e, "<->"))) then 
    let $x := tokenize($e, "->")!normalize-space() 
    return map {xs:integer($x[1]) + $indexOffset: xs:integer($x[2]) + $indexOffset}
  else ()
);


declare variable $bidi-map := local:edge-list-to-maps(
  for $e in $edges return
  if (contains($e, "<->")) then 
    let $x := tokenize($e, "<->")!normalize-space() 
    return (map {xs:integer($x[1]) + $indexOffset: xs:integer($x[2]) + $indexOffset}, 
            map {xs:integer($x[2]) + $indexOffset: xs:integer($x[1]) + $indexOffset})
  else ()
);

declare variable $directed-matrix := 
  for $j in $ids return array {
    for $i in $ids return 
      if ($i eq $j) then "1"
      else if ($directed-map?($i)?($j)) then "-" || $name-lambda-prefix || $i || $name-separator || $j
      else 0
  };
  
declare variable $bidi-matrix := 
  for $i in $ids return array {
    for $j in $ids return 
      if ($i eq $j) then "W" || $i
      else if ($bidi-map?($i)?($j)) then (
        if ($i < $j) then $name-omega-prefix|| $i || $name-separator || $j
        else $name-omega-prefix|| $j || $name-separator || $i
      ) else 0
  };

declare function local:sigma-matrix(){
  for $i in $ids return array {
    for $j in $ids return 
      if ($i eq $j) then "S" || $i
      else if ($i < $j) then $name-sigma-prefix|| $i || $name-separator || $j
      else $name-sigma-prefix || $j || $name-separator || $i
  }
};

  
switch ($outputFormat) 
  case "maxima" 
return 
(
"""input: "||$edgesInput||""";",
"IML: matrix (" || (string-join($directed-matrix ! concat("[", string-join(., ", "), "]"), ", " ) ) || ");",
"W: matrix (" || (string-join($bidi-matrix ! concat("[", string-join(., ", "), "]"), ", " ) ) || ");",
"",
"Sigma: ratsimp(invert(IML).W.transpose(invert(IML)));",
"",
"SimpleSigma: "||string-join($ids ! concat("subst(-(Sigma[",.,",",.,"])+1+W",.,", W"||.,"," )) || "Sigma" || string-join($ids!")")||";",
"",
"S: matrix (" || (string-join(local:sigma-matrix() ! concat("[", string-join(., ", "), "]"), ", " ) ) || ");",
"ratsimp(IML.S.transpose(IML));"
)

 case "debug" 
return 
(
$directed-matrix,
$bidi-matrix

)

default return ()
