xquery version "3.1-xidel";
import module namespace helper = "identification-helper" at "identification-helper.xqm";

(: declare variable $model := "[[1, 3], [2, 3], [2, 4], [1, 4]]"; :)
(: declare variable $model external := "[[1, 3], [3, 2], [2, 4], [ 1,4]]"; :)
 declare variable $model external := "[[1, 3], [3, 2], [2, 4], [ 4,1]]"; 
(: declare variable $model := "[[1, 2], [2, 3], [3, 4], [1, 4]]";  some id :)
declare variable $n as xs:integer external := count($missing-edges-unordered);

declare variable $cycle external := $model;

declare variable $graph external := if ($model != $cycle) then trace( helper:parse-model($model), "graph") else ();

declare variable $first-node-index-offset as xs:integer external := 0;



declare variable $missing-edges-unordered := 
  if ($model = "abcd") then "abcd"
  else if ($cycle != $model) then 
    if (matches($cycle, "\[ *\[")) then parse-json($cycle)?*
    else let $c := tokenize($cycle)[normalize-space()]
         for $e at $i in $c
         return [$e, ($c[$i+1], $c[1])[1]]
  else parse-json(if ($model ne "-") then $model else read())();


declare variable $missing-cycle :=
  if ($model = "abcd") then "abcd"
  else
  let $wrap-around := ($missing-edges-unordered, $missing-edges-unordered[1])
                                   for $e at $i in $missing-edges-unordered
                                   let $e := $e()
                                   let $next := $wrap-around[$i + 1]()
                                   return $e[ not(. = $next) ];
                                   

declare variable $missing-edges := let $wrap-around := ($missing-cycle, $missing-cycle[1])
                                   for $x at $i in $missing-cycle
                                   return [$x, $wrap-around[$i+1]];

declare variable $SIGMA external  := "σ" ;

declare function make-sigma($i, $j){
  let $t := ($i + $first-node-index-offset, $j + $first-node-index-offset)
  return $SIGMA || min($t) || max($t)
};

declare function make-neg-sigma($i, $j){
  "(- " || make-sigma($i, $j) || ")"
};

declare function get-parent($i){
  if ($cycle = $model) then $i - 1
  else exactly-one($graph("->")[.(2) = $i](1))
};

declare variable $sigmas := 
  if ($model = "abcd") then 
    (1 to $n) ! ["a" || ., "b" || ., "c" || ., "d" || .]
  else for $e in $missing-edges
       let $i := $e()[1]
       let $j := $e()[2]
       return [ make-sigma(get-parent($i), get-parent($j)), make-neg-sigma(get-parent($i), $j), make-neg-sigma($i,get-parent($j)), make-sigma($i, $j) ];



declare variable $max-level := ceiling(math:log($n) div math:log(2)) + 1;

declare variable $level-size := (1 to $max-level) ! level-size(.);

declare function level-size($level){
  if ($level = 1) then $n
  else let $ls := level-size($level - 1) 
  return ($ls + 1) idiv 2
};

declare function det($m11, $m12, $m21, $m22){
  x"({$m11} * {$m22} - {$m12} * {$m21})"
};

declare function a($i, $level) {
  if ($level eq 1) then $sigmas[$i](1)
  else 
  let $index2i := 2*$i 
  let $index2i-1 := $index2i - 1
  let $l := $level - 1
  return
  if ($index2i > $level-size[$l] ) then a($index2i-1, $l)
  else det( a($index2i-1, $l), a($index2i, $l), b($index2i-1, $l), c($index2i, $l) )
};

declare function b($i, $level) {
  if ($level eq 1) then $sigmas[$i](2)
  else 
  let $index2i := 2*$i 
  let $index2i-1 := $index2i - 1
  let $l := $level - 1
  return
  if ($index2i > $level-size[$l] ) then b($index2i-1, $l)
  else det( a($index2i-1, $l), b($index2i, $l), b($index2i-1, $l), d($index2i, $l) )
};

declare function c($i, $level) {
  if ($level eq 1) then $sigmas[$i](3)
  else 
  let $index2i := 2*$i 
  let $index2i-1 := $index2i - 1
  let $l := $level - 1
  return
  if ($index2i > $level-size[$l] ) then c($index2i-1, $l)
  else det( c($index2i-1, $l), a($index2i, $l), d($index2i-1, $l), c($index2i, $l) )
};

declare function d($i, $level) {
  if ($level eq 1) then $sigmas[$i](4)
  else 
  let $index2i := 2*$i 
  let $index2i-1 := $index2i - 1
  let $l := $level - 1
  return
  if ($index2i > $level-size[$l] ) then d($index2i-1, $l)
  else det( c($index2i-1, $l), b($index2i, $l), d($index2i-1, $l), d($index2i, $l) )
};

declare function prefix($i, $l, $c){
  $c || ": "
};

$missing-edges,
join($missing-cycle),
$sigmas,
for $i in 1 to count($sigmas) 
let $s := $sigmas[$i]?*
let $a := $missing-cycle[$i]
let $b := ($missing-cycle, $missing-cycle[1])[$i + 1]
return x"l{$a}*l{$b}*{$s[1]} + l{$a}*{$s[2]} + l{$b}*{$s[3]} + {$s[4]}",
string-join($level-size, " "),
for $l in 1 to $max-level ,
    $i in 1 to $level-size[$l]
return (
"","","","LEVEL " || $l || " " || $i,
a( $i, $l ) || ";", 
b( $i, $l ) || ";",
c( $i, $l ) || ";",
d($i, $l) || ";"
),
"","FINAL","",
prefix(1, $max-level, "a") || a( 1, $max-level ) || ";", 
prefix(1, $max-level, "b") || b( 1, $max-level ) || " + " || c( 1, $max-level ) || ";",
prefix(1, $max-level, "c") || d(1, $max-level) || ";"

