xquery version "3.1-xidel";
module namespace helper = "identification-helper";

declare function helper:parse-model($model, $flags){
  if (matches($model, "\[ *\[")) then helper:parse-json-model($model)
  else if (contains($model, "<->")) then helper:parse-pretty-edge-model($model, $flags)
  else  error("invalid model: " || $model)
};
declare function helper:parse-model($model){ helper:parse-model($model, {}) };

declare function helper:present-edges-to-missing-edges($node-count, $edges){

  $edges ! array {
    let $edges := .?*
    let $existence-map := map:merge($edges ! map { min(.?*) || "x" || max(.?*): true() })
    for $a in 0 to $node-count - 1 
    for $b in $a + 1 to $node-count - 1 
    let $key := $a || "x" || $b
    where not($existence-map($key))
    return [$a, $b]
  }
};

declare function helper:parse-pretty-edge-model($model, $flags){
  let $edges := (tokenize($model)!normalize-space()[.])
  let $edges-ids := (pxp:extract($model, "[0-9]+", 0, "*")!xs:integer(.))
  let $had-0 := $edges-ids = 0
  
  let $first-node-index := ($flags?first-node-index, if ($had-0) then 0 else 1 )[1]
  let $n := max($edges-ids) + 1 - $first-node-index 
  let $last := $n - 1 + $first-node-index
  let $offset := if ($first-node-index = 1 and $had-0) then 1 else 0
  return map {
    "first-node-index": $first-node-index,
    "last-node-index": $last,
    "n": $n,
    "->": for $e in $edges[contains(., "->") and not(contains(., "<->"))] let $t := tokenize($e, "->")!xs:integer(.)!(.+$offset) return [$t[1],$t[2]],
    "<->": for $e in $edges[contains(., "<->")] let $t := tokenize($e, "<->")!xs:integer(.)!(.+$offset) return [min($t),max($t)]
  }
};

declare function helper:parse-json-model($model){
  let $first-node-index := 0 
  let $shift := 0
  
  let $json := parse-json($model)
  let $n := max ( $json ?* ?* ) + 1 (:max(pxp:extract($model, "[0-9]+", 0, "*")!xs:integer(.)):)
  let $last := $n - ( 1 - $first-node-index )
  return map {
    "first-node-index": $first-node-index,
    "last-node-index": $last,
    "n": $n,
    "->": ($first-node-index to $last - 1) ! [ ., . + 1 ],
    "<->": helper:present-edges-to-missing-edges($n, $json)  ?*  ! [?1 + $shift, ?2 + $shift]
  }
};

declare function helper:json-model-to-pretty-edge-string($model){
  let $model := helper:parse-json-model($model)
  return 
  string-join( ( $model("->") ! (?1 || "->" || ?2), $model("<->") ! (?1 || "<->" || ?2) )  , " ")
};

declare function helper:json-model-to-dagitty($model){
  let $model := helper:parse-json-model($model)
  return 
  string-join( ("dag{",
                $model("->") ! ("  " ||?1 || "->" || ?2), 
                $model("<->") ! ("  " ||?1 || "<->" || ?2)
                , "}" )  , "&#xA;")
};


declare variable $helper:drton-model-regex := "[(] *([0-9]+) *, *([0-9]+) *[)]";

declare function helper:drton-model-to-pretty-edge-string($n, $model){
  let $temp := extract($model, $helper:drton-model-regex, (1,2))
  let $d := $temp[1]
  let $b := $temp[2]
  let $d2 := x:integer-to-base($d => xs:integer(), 2) => string-to-codepoints() => reverse()
  let $b2 := x:integer-to-base($b => xs:integer(), 2) => string-to-codepoints() => reverse()
  return (
    for $v in 0 to $n - 1
    for $w in 0 to $n - 1
    where $v ne $w
    count $idx
    where $d2[$idx] = 49
    return $v || "->" || $w,

    for $v in 0 to $n - 2
    for $w in $v + 1 to $n - 1
    count $idx
    where $b2[$idx] = 49
    return $v || "<->" || $w

  )
    
};